#ifndef DEVICE_H
#define DEVICE_H

#include "MessageList.h"
#include <iostream>
#include <queue>
#include "Utility.h"
namespace rwil {
  namespace ExecuteReturns{
    /// This enumeration represents whether or not to retry a particular message
    enum Returns { PleaseRetry = 0, DontRetry };
  }

  /// Represents an abstract device capable of accepting messages
  /**
    * A Device is an abstract class representing the concept of a device that can accept
    * messages and act upon them.
    */
  class Device
    {
    public:
      /// Create a device with a specified name and linked to a specific output queue
      /**
        * @param name The name of the device. This can be anything
        * @param output The output MessageList that the device is linked to. This MessageList will recieve all error and success messages from the device.
        */
      Device(const std::string &name, MessageList& output);

      virtual ~Device();

      /// Return a device-type dependent value representing the status of the device.
      /**
        * @return The status in a string form.
        */
      virtual std::string GetStatus() const = 0;

      /// Get the name of the device.
      /**
        * @return The string name of the device specified in the constructor
        */
      const std::string &GetName() const;

      /// Add a message to the execute queue for this device.
      /**
        * @param msg The message to add to this device. This message MUST be associated with this device.
        */
      void AddToQueue(const Message &msg);


      /// Execute the top message in the queue.
      /**
        * @return A boolean value representing whether or not ANY action is attempted
        */
      bool Tick();

    protected:
      /// The output MessageList, this is where success and failure messages are sent
      MessageList &m_output;

      /// The function which performs the messages.
      /**
        * This function is written so that it returns PleaseRetry if it wants the message to be
        * sent to it again. Messages are only repeated a certain number of times(default: 3). This
        * can be changed by send a SetRetriesMessage to the device.
        * @param msg Message to execute
        * @return Whether or not the message should be retried
        */
      virtual ExecuteReturns::Returns ExecuteMessage(const Message & msg) = 0;

    private:
      /// The string name of the device
      const std::string m_name;
      /// The execution queue of messages.
      std::queue<Message> m_queue;
      /// The number of retries to perform
      unsigned m_retries;
      /// The number of retries that have been performed on the current message
      unsigned m_currentRetry;
    };
}
#endif
