#include <string>
#include <map>

#ifndef MESSAGE_H
#define MESSAGE_H
namespace rwil {

  /// Generic Message class. Uses strings to represent parameters and values. Additionally keeps a timestamp
  class Message
    {
    public:
      /// Initialize a message with device and command parameters the default timestamp of now
      /**
        * @param deviceName Device to send message to
        * @param command The command to send
        */
      Message(std::string deviceName, std::string command);
      /// Initialize a message with device and command parameters with a specified timestamp
      /**
        * @param deviceName Device to send message to.
        * @param command The command to send.
        * @param when The timestamp to give the message.
        */
      Message(std::string deviceName, std::string command, time_t when);
      /// Initialize a message with a map of parameters and the default timestamp of now
      /**
        * @param params A map with the various parameters for the message specified.
        */
      Message(std::map<std::string,std::string> params);
      /// Initialize a message with a map of parameters and a specified timestamp
      /**
        * @param params A map with the various parameters for the message specified.
        * @param when The timestamp to give the message
        */
      Message(std::map<std::string,std::string> params, time_t when);
      /// Copy constructor
      /**
        * @param msg The message to copy.
        */
      Message(const Message &msg);

      /// Get the value of the "device" parameter
      /**
        * @return The value of the "device" parameter.
        */
      const std::string& GetDevice() const;
      /// Get the value of the "command" parameter
      /**
        * @return The value of the "command" parameter.
        */
      const std::string& GetCommand() const;

      /// Less than operator used to maintain order in the MessageList class
      /**
        * Return true if the timestamp is less than that of rhs's
        * @param rhs The variable on the right hand side of the operation.
        * @return Whether or not the timestamp is less.
        */
      bool operator<(const Message& rhs);

      /// Sets the value of a parameter
      /**
        * @param n The name of the parameter
        * @param param The new value of the parameter
        */
      void SetParam(std::string n, std::string param);
      /// Gets the value of a parameter.
      /**
        * If the parameter is not defined then returns ""
        * @param n The name of the parameter
        * @return The value of that named parameter. Returns "" for non-existant parameters.
        */
      std::string GetParam(std::string n) const;
      /// Gets the timestamp associated with the message
      /**
        * Get the value of the timestamp
        * @return The timestamp as a time_t value
        */
      time_t GetWhen() const;
      /// Gets the internal data structure used to maintain state.
      /**
        * @return A constant reference to the internal structure.
        */
      const std::map<std::string,std::string>& GetMap() const;

    protected:
    private:
      /// The parameters of the message
      mutable std::map<std::string,std::string> m_parameters;
      /// The message timestamp
      const time_t m_when;
    };

  /// Creates a message containing all the data of the old message and a new parameter "message"
  class EncapsulatingMessage : public Message
    {
    public:
      /// Creates a message with a new parameter "message" and the data from an old message the timestamp is set to now
      /**
        * @param encapsulateCommand The value of the new parameter named "message"
        * @param oldMessage Message with old data to copy.
        */
      EncapsulatingMessage(std::string encapsulateCommand, const Message &oldMessage);
      /// Creates a message with a new parameter "message" and the data from an old message. The timestamp is set the when.
      /**
        * @param encapsulateCommand The value of the new parameter named "message"
        * @param oldMesage Message with the old data to copy
        * @param when Timestamp of the message
        */
      EncapsulatingMessage(std::string encapsulateCommand, const Message & oldMessage, time_t when);
    };

  /// This message type represents a message with an error because of a failure after retrying many many times.
  class FailedAfterRetryingMessage : public EncapsulatingMessage
    {
    public:
      /// Contains the old message that failed, and a timestamp of now, the new data is a parameter "message" with the value "Failed after retrying message"
      /**
        * @param oldMessage Message with old data
        */
      FailedAfterRetryingMessage(const Message & oldMessage);
    };

  /// A message for setting the number of retries.
  class SetRetriesMessage : public Message
    {
    public:
      /// Creates a message setting the number of retries.
      /**
        * @param deviceName The name of the device on which to set the number of retries.
        * @param retries The number of retries to perform.
        */
      SetRetriesMessage(std::string deviceName, unsigned retries);
    };
}
#endif
