#ifndef MESSAGELIST_H
#define MESSAGELIST_H
#include <list>
#include "Message.h"
#include "Synchronization.h"
namespace rwil {
  typedef void (*rwil_MessageList_AddCallback_t)(const list<Message>&);

  /// Maintains a thread-safe list of messages
  class MessageList
    {
    public:
      /// Adds a single message to the list
      /**
        * @param msg Message to add to the list. The callback is called before the message is added.
        */
      void Add(const Message& msg);
      /// Adds several messages to the list
      /**
        * @param messages Messages to add to the list. The callback is called before any messages are added.
        */
      void Add(list<Message>& messages);
      /// Sets a Callback that is called before an add takes place. The callback function must be of type void(*func)(const list<Message&)
      /**
        * @param callback The callback that is called before an add takes place. The callback recieves a parameter of type const list<Message&> which represents the messages that are about to be added.
        **/
      void SetAddCallback(rwil_MessageList_AddCallback_t callback);
      /// Query for a list of messages that are timestamped between start and end inclusive.
      /**
        * @param start The start of the span of time to query in.
        * @param end The end of the span of time to query in.
        * @return A list of pointers to Message's that fit the query.
        */
      list<Message *> Query(time_t start, time_t end);
      /// Query for all messages that have a parameter with a specific value
      /**
        * @param paramname The name of the parameter to examine.
        * @param value The value of paramname to match.
        * @return A list of pointers to Message's that fit the query.
        */
      list<Message *> Query(std::string paramname, std::string value);
      /// Removes a list of Messages from the list.
      /**
        * The list of messages to remove must be acquired from the Query functions in some way.
        * @param messages The list of messages to remove from the list.
        */
      void Destroy(list<Message*> messages);
      /// Locks the List
      /**
        * This function is designed to be used so that modifications to the list
        * that use more than one function can be executed without interference
        * from other threads.
        */
      void Lock();
      /// Unlocks the list
      /**
        * This function is designed to be used so that modifications to the list
        * that use more than one function can be executed without interference
        * from other threads.
        */
      void Release();
      /// Waits for a message to be added.
      /**
        * This may return immediately if an item was added to the list and not waited on previously. This function has a timeout measure in milliseconds.
        * @param milli number of milliseconds to wait
        */
      bool Wait(int milli);
      /// Waits for a message to be added.
      /**
        * This may return immediately if an item was added to the list and not waited on previously.
        */
      void Wait();
    private:
      /// The list of messages inside the MessageList
      list<Message> m_messages;
      /// The mutex that ensures thread-safety
      Mutex m_op;
      Mutex m_lock;
      /// The semaphore used to implement the waiting functions
      Semaphore m_count;
      /// The add callback
      rwil_MessageList_AddCallback_t m_addcallback;
    };
}
#endif

