#ifdef LINUX
#include <pthread.h>
#endif

#ifndef THREAD_H
namespace rwil {
  /// A mutex class.
  /**
    * This is intended to be a cross-platform class; however,
    * only the linux implementation has been used. Therefore
    * the windows version probably doesn't actually work or compile
    * so it needs to be worked on. This is a recursive mutex.
    */
  class Mutex
    {
    public:
      /// Create a mutex
      /**
        * Creates a mutex that is not already held.
        */
      Mutex();
      /// Destroy a mutex
      /**
        * Release all resources associated with this mutex.
        */
      ~Mutex();

      /// Wait on a mutex until it is available.
      /**
        * This is a blocking call. If a different thread has
        * the mutex, then this will wait until that thread releases.
        */
      void Wait();
      /// Try to wait on a mutex
      /**
        * This returns immediately.
        * @return True if the mutex is acquired, false if not
        */
      bool TryWait();
      /// Release the mutex
      /**
        * Unpredictable results occur when the mutex is not held
        * before this call.
        */
      void Release();

    private:
#ifdef _WIN32
	  /// The os-dependent handle for the mutex in Windows(tm)
      HANDLE m_mutex;
#endif
#ifdef LINUX
      /// The os-dependent handle for the mutex in POSIX
      pthread_mutex_t m_mutex;
#endif
    };

  /// Creates a mutex that automatically Waits and Releases a mutex
  /**
    * This class waits on the mutex specified in the constructor
    * and releases that mutex when it destructs.
    */
  class ScopedMutex
    {
    public:
      /// Create a scoped mutex associated with a particular mutex.
      /**
        * This call waits on the specified mutex immediately.
        * @param mutex The mutex to associate with the scope.
        */
      ScopedMutex(Mutex & mutex);
      /// Destroys the scoped mutex.
      /**
        * This call releases the mutex specified in the constructor.
        */
      ~ScopedMutex();
    private:
      /// A reference to the mutex passed to the costructor.
      Mutex &m_mutex;
    };

  /// Creates a critical section
  /**
    * This is used to make it so that a particular section of code is
    * only by one thread at a time. This implemented using a mutex.
    */
  class CriticalSection
    {
    public:
      /// Constructs a critical section
      CriticalSection() { m_mutex.Wait(); }
      /// Destructs a critical section
      ~CriticalSection() { m_mutex.Release(); }
    private:
      /// Mutex used internally to implement critical section.
      Mutex m_mutex;
    };

  /// Creates a semaphore
  /**
    * This is a general-purpose cross-platform counting semaphore.
    */
  class Semaphore
    {
    public:
      /// Create a semaphore with a given initial value
      /**
        * When the count is greater than 0 a Wait call will succeed.
        * @param count The initial value of the semaphore
        */
      Semaphore(int count);
      /// Create a semaphore with an initial value of 0
      /**
        * The semaphore must be signalled at least once before
        * a wait call will be successful.
        */
      Semaphore();

      /// Wait functions will return successfully when the count is greater than one
      //  They then decrement the count
      /// Blocking Wait on the semaphore
      /**
        * The semaphore will block until its value is > 0. Then the semaphore value
        * will be decremented and the function will return.
        */
      void Wait();
      /// Non-blocking Wait on semaphore
      /**
        * The semaphore will check to see if its value is > 0. If so, it will be decremented
        * and the semaphore will return true.
        * @return True if the wait was successsful, False if not.
        */
      bool TryWait();
      /// Blocking wait on semaphore with timeout
      /**
        * This semaphore will block on its value for at least millisecondTimeout milliseconds
        * or until the value of semaphore becomes > 0. If it becomes > 0 the semaphore
        * will be decremented and the semaphore will return true.
        * @param millisecondTimeout The number of milliseconds to block before returning false.
        * @return True if the semaphore was decremented, false if it timed out.
        */
      bool Wait(int millisecondTimeout);
      /// Signal the semaphore.
      /**
        * Increment the semaphore. When it becomes > 0, if a thread is waiting on it, that thread
        * will be released.
        */
      void Signal();
    private:
      /// Mutex used internally to maintain thread safety.
      Mutex m_mutex;
      /// The semaphore's value
      int m_count;
    };

  ///Creates a thread which allows one instance of a thread at a time to execute
  /**
    * This class encapsulates a single thread of execution. Call Start() to start the thread.
    */
  class Thread
    {
    public:
      /// Constructs a thread.
      /**
        * Essentially does nothing.
        */
      Thread();
      /// Destructs a thread.
      /**
        * Ensures that the mutex is not held anymore.
        */
      virtual ~Thread();
      /// Start the thread.
      /**
        * Begins the thread of execution. Ensures that this thread does not start
        * again until this one dies.
        */
      void Start();
      /// Begins the execution of the function in THIS thread.
      /**
        * This function is used internally to start execution in the new thread of execution.
        */
      void Go();
      /// Join the thread
      /**
        * Wait for the thread to finish.
        */
      void Join();
    protected:
      /// Yield timeslice.
      /**
        * This should be used inside of the thread instead of something os-dependent.
        */
      void Yield();
      /// This is the function that implements the thread of execution.
      /**
        * This function must be implemented by all deriving classes.
        */
      virtual void Function() = 0;
      /// Mutex used to ensure that this is only executing one thread of execution.
      Mutex m_mutex;
#ifdef _WIN32
      /// Windows(tm) os-dependent thread handle
      HANDLE m_threadHandle
#endif
#ifdef LINUX
    /// POSIX os-dependent thread handle
	pthread_t m_threadHandle;
#endif
    };
}
#define THREAD_H
#endif

