#include "rwil.h"
#include "procrwil.h"
#include "X10Device.h"
#include <stdio.h>
using namespace rwil;


static void rwil_callback(const list<Message>& messages);


/// Initialize the Real World Interface Library
rwil_handle rwil_init()
{
  RWIL *ptr = new RWIL;
  rwil_handle handle;
  ptr->Start();
  handle = ptr;
  ptr->output.SetAddCallback(rwil_callback);
  return handle;
}

/// Create a device in the RWIL with the specified attributes
rwil_device rwil_create(rwil_handle rwilhandle, char house, int num)
{
  char buf[20];
  int val;
  RWIL *rwil = (RWIL*)rwilhandle;
  rwil_device device;
  val = (house << 16) + num;
  sprintf(buf, "%d", val);
  X10Device *dev = new X10Device(buf, rwil->output, house, num);
  rwil->AddDevice(dev);
  device.handle = rwilhandle;
  device.dev_handle = val;
  return device;
}

/// Destroy a device from the RWIL with the specified handle
void rwil_destroy(rwil_device handle)
{
  RWIL *rwil = (RWIL*)handle.handle;
  char buf[20];
  sprintf(buf, "%d", handle.dev_handle);
  rwil->RemoveDevice(buf);
}

/// Turn a device on to full power
void rwil_on(rwil_device handle, time_t when)
{
  rwil_setbright(handle, 100, when);
}
/// Turn a device off
void rwil_off(rwil_device handle, time_t when)
{
  rwil_setbright(handle, 0, when);
}

/// Set the brightness of a device
void rwil_setbright(rwil_device handle, int brightness, time_t when)
{
  RWIL *rwil = (RWIL*)handle.handle;
  char buf[20];
  sprintf(buf, "%d", handle.dev_handle);

  Message msg(buf, "dim", when);
  msg.SetParam("percent", IntToString(brightness));
  msg.SetParam("rwilhandle", IntToString((int)rwil));
  rwil->input.Add(msg);
}

/// Interpolate brightness between two values over a time interval
void rwil_interpolatebright(rwil_device handle, int startbrightness, int endbrightness, time_t startwhen, time_t endwhen, int steps)
{
  RWIL *rwil = (RWIL*)handle.handle;
  char buf[20];

  time_t timedif = endwhen - startwhen;
  int brightdif = endbrightness - startbrightness;

  sprintf(buf, "%d", handle.dev_handle);

  int count = 0;
  for(count = 0; count < steps; count++)
  {
    Message msg(buf, "dim", startwhen + timedif * count / steps);
    msg.SetParam("percent", IntToString(startbrightness + brightdif * count / steps));

    //Ensure that the callback is only called by the last change in brightness
    if((count + 1)== steps)msg.SetParam("rwilhandle", IntToString((int)rwil));
    rwil->input.Add(msg);
  }
}

/// Get the brightness of a device now
int rwil_getbright(rwil_device handle)
{
  RWIL *rwil = (RWIL*)handle.handle;
  char buf[20];
  sprintf(buf, "%d", handle.dev_handle);

  X10Device* dev = (X10Device *)rwil->GetDevice(buf);
  return StringToInt(dev->GetStatus());
}

/// Determine whether the device is on now
int rwil_ison(rwil_device handle)
{
  if(rwil_getbright(handle)) return 1;
  return 0;
}
static rwil_success_callback_t success_call = NULL;
static rwil_failure_callback_t failure_call = NULL;

/// Set the success callback
void rwil_success_callback(rwil_success_callback_t callback)
{
  success_call = callback;
}

/// Set the failure callback
void rwil_failure_callback(rwil_failure_callback_t callback)
{
  failure_call = callback;
}

/// Shutdown the RWIL this may take several seconds and is a blocking call
void rwil_shutdown(rwil_handle rwilhandle)
{
  RWIL *rwil = (RWIL*)rwilhandle;
  rwil->Kill();
  rwil->Join();
  delete rwil;
}

static void rwil_callback(const list<Message>& messages)
{
  list<Message>::const_iterator iter;

  iter = messages.begin();
  rwil_device dev;
  while(iter != messages.end())
    {
      std::string str = iter->GetParam("rwilhandle");
      if(str=="")
	{
	  iter++;
	  continue;
	}
      dev.handle = (void *)StringToInt(str);
      dev.dev_handle = StringToInt(iter->GetDevice());
      str = iter->GetParam("message");
      if(str == "success")
	{
	  if(success_call)
	    {
	      success_call(dev);
	    }
	}
      else
	{
	  if(failure_call)
	    {
	      failure_call(dev, str.c_str());
	    }
	}
      iter++;
    }
}
