#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif
  typedef void * rwil_handle;
  typedef struct { rwil_handle handle; int dev_handle; } rwil_device;

  /// Initialize the Real World Interface Library
  /**
    * Call this before any other functions.
    */
  rwil_handle rwil_init();

  /// Create a device in the RWIL with the specified attributes
  /**
    * @param rwilhandle The value returned from rwil_init()
    * @param house The letter of the house address, a-p or A-P
    * @param num The unit number address 1-16
    * @return A handle to the device.
    */
  rwil_device rwil_create(rwil_handle rwilhandle, char house, int num);

  /// Destroy a device from the RWIL with the specified handle
  /**
    * @param handle Device to destroy.
    */
  void rwil_destroy(rwil_device handle);

  /// Turn a device on to full power
  /**
    * @param handle Device to turn on.
    * @param when When to turn on.
    */
  void rwil_on(rwil_device handle, time_t when);

  /// Turn a device off
  /**
    * @param handle Device to turn off.
    * @param when When to turn device off.
    */
  void rwil_off(rwil_device handle, time_t when);

  /// Set the brightness of a device
  /**
    * @param handle Device to set brightness on.
    * @param brightness Level to set brightness to.
    * @when When to set brightness
    */
  void rwil_setbright(rwil_device handle, int brightness, time_t when);

  /// Interpolate brightness between two values over a time interval
  /**
    * Callback is only called at completion or failure of last event.
    * @param handle Device to set brightness of.
    * @param startbrightness The starting brightness value
    * @param endbrightness The ending brightness value
    * @param startwhen The time to start inerpolating from startbrightness to endbrightness
    * @param endwhen The time for the command to finish
    * @param steps The number of steps to take in the interpolations.
    */
  void rwil_interpolatebright(rwil_device handle, int startbrightness, int endbrightness, time_t startwhen, time_t endwhen, int steps);

  /**
    * Get the current brightness of a device.
    * @param handle The device to check the brightness of.
    */
  /// Get the brightness of a device now
  int rwil_getbright(rwil_device handle);

  /// Determine whether the device is on now
  /**
    * @param handle The device to check the status of.
    */
  int rwil_ison(rwil_device handle);

  /// The type used for success callbacks.
  /**
    * This function takes as a parameter the device handle of the device that succesfully executed.
    */
  typedef void (*rwil_success_callback_t)(rwil_device);

  /// Set the success callback
  /**
    * Set the global success callback.
    * @param callback A function pointer to the callback function.
    */
  void rwil_success_callback(rwil_success_callback_t callback);

  /// The type used for failure callbacks
  /**
    * This function takes as a parameter the device handle of the device that unsuccesfully executed, and a textual error message.
    */
  typedef void (*rwil_failure_callback_t)(rwil_device, const char *);

  /// Set the failure callback
  /**
    * Set the global failure callback.
    * @param callback A function pointer to the callback function.
    */
  void rwil_failure_callback(rwil_failure_callback_t callback);

  /// Shutdown the RWIL
  /**
    * This function shuts down the rwil system, and waits for the shutdown to complete. This could take several seconds.
    * @param rwilhandle Handle to the RWIL system as returned by rwil_init()
    */
  void rwil_shutdown(rwil_handle rwilhandle);

#ifdef __cplusplus
}
#endif
